pub struct StorageCtx;Expand description
Thread-local storage accessor that implements PrecompileStorageProvider without the trait bound.
This is the only type that exposes access to the thread-local STORAGE static.
Important
Since it provides access to the current thread-local storage context, it MUST be used within
a StorageCtx::enter closure.
Sync with PrecompileStorageProvider
This type mirrors PrecompileStorageProvider methods but with split mutability:
- Read operations (staticcall) take
&self - Write operations take
&mut self
Implementations
Sourceimpl StorageCtx
impl StorageCtx
Sourcepub fn enter<S, R>(storage: &mut S, f: impl FnOnce() -> R) -> Rwhere
S: PrecompileStorageProvider,
pub fn enter<S, R>(storage: &mut S, f: impl FnOnce() -> R) -> Rwhere
S: PrecompileStorageProvider,
Enter storage context. All storage operations must happen within the closure.
IMPORTANT
The caller must ensure that:
- Only one
entercall is active at a time, in the same thread. - If multiple storage providers are instantiated in parallel threads, they CANNOT point to the same storage addresses.
Sourcefn with_storage<F, R>(f: F) -> Rwhere
F: FnOnce(&mut dyn PrecompileStorageProvider) -> R,
fn with_storage<F, R>(f: F) -> Rwhere
F: FnOnce(&mut dyn PrecompileStorageProvider) -> R,
Execute an infallible function with access to the current thread-local storage provider.
Panics
Panics if no storage context is set.
Sourcefn try_with_storage<F, R>(f: F) -> Result<R>
fn try_with_storage<F, R>(f: F) -> Result<R>
Execute a (fallible) function with access to the current thread-local storage provider.
Sourcepub fn with_account_info<T>(
&self,
address: Address,
f: impl FnMut(&AccountInfo) -> Result<T>,
) -> Result<T>
pub fn with_account_info<T>( &self, address: Address, f: impl FnMut(&AccountInfo) -> Result<T>, ) -> Result<T>
Executes a closure with access to the account info, returning the closure’s result.
This is an ergonomic wrapper that flattens the Result, avoiding double ?.
Sourcepub fn beneficiary(&self) -> Address
pub fn beneficiary(&self) -> Address
Returns the current block beneficiary (coinbase).
Sourcepub fn block_number(&self) -> u64
pub fn block_number(&self) -> u64
Returns the current block number.
Sourcepub fn set_code(&mut self, address: Address, code: Bytecode) -> Result<()>
pub fn set_code(&mut self, address: Address, code: Bytecode) -> Result<()>
Sets the bytecode at the given address.
Sourcepub fn sload(&self, address: Address, key: U256) -> Result<U256>
pub fn sload(&self, address: Address, key: U256) -> Result<U256>
Performs an SLOAD operation (persistent storage read).
Sourcepub fn tload(&self, address: Address, key: U256) -> Result<U256>
pub fn tload(&self, address: Address, key: U256) -> Result<U256>
Performs a TLOAD operation (transient storage read).
Sourcepub fn sstore(&mut self, address: Address, key: U256, value: U256) -> Result<()>
pub fn sstore(&mut self, address: Address, key: U256, value: U256) -> Result<()>
Performs an SSTORE operation (persistent storage write).
Sourcepub fn tstore(&mut self, address: Address, key: U256, value: U256) -> Result<()>
pub fn tstore(&mut self, address: Address, key: U256, value: U256) -> Result<()>
Performs a TSTORE operation (transient storage write).
Sourcepub fn emit_event(&mut self, address: Address, event: LogData) -> Result<()>
pub fn emit_event(&mut self, address: Address, event: LogData) -> Result<()>
Emits an event from the given contract address.
Sourcepub fn refund_gas(&mut self, gas: i64)
pub fn refund_gas(&mut self, gas: i64)
Adds refund to the gas refund counter.
Sourcepub fn gas_refunded(&self) -> i64
pub fn gas_refunded(&self) -> i64
Returns the gas refunded so far.
Sourcepub fn spec(&self) -> TempoHardfork
pub fn spec(&self) -> TempoHardfork
Returns the currently active hardfork.
Sourcepub fn checkpoint(&mut self) -> CheckpointGuard
pub fn checkpoint(&mut self) -> CheckpointGuard
Creates a journal checkpoint and returns a RAII guard.
All state mutations after this call will be atomically
reverted if the guard is dropped without calling
CheckpointGuard::commit.
Panics
Panics if no storage context is set.
Sourcepub fn deduct_gas(&mut self, gas: u64) -> Result<()>
pub fn deduct_gas(&mut self, gas: u64) -> Result<()>
Deducts gas from the remaining gas and returns an error if insufficient.
Sourcepub fn keccak256(&self, data: &[u8]) -> Result<B256>
pub fn keccak256(&self, data: &[u8]) -> Result<B256>
Computes keccak256 and charges the appropriate gas.
Prefer this over naked keccak256 to ensure gas is accounted for.
Sourcepub fn recover_signer(
&self,
digest: B256,
v: u8,
r: B256,
s: B256,
) -> Result<Option<Address>>
pub fn recover_signer( &self, digest: B256, v: u8, r: B256, s: B256, ) -> Result<Option<Address>>
Recovers the signer address from an ECDSA signature and charges ecrecover gas.
As per TIP-1004, it only accepts v values of 27 or 28 (no 0/1 normalization).
Returns Ok(None) on invalid signatures; callers map to domain-specific errors.
Sourceimpl<'evm> StorageCtx
impl<'evm> StorageCtx
Sourcepub fn enter_evm<J, R>(
journal: &'evm mut J,
block_env: &'evm dyn Block,
cfg: &CfgEnv<TempoHardfork>,
tx_env: &'evm impl Transaction,
f: impl FnOnce() -> R,
) -> Rwhere
J: JournalTr<Database: Database> + Debug,
pub fn enter_evm<J, R>(
journal: &'evm mut J,
block_env: &'evm dyn Block,
cfg: &CfgEnv<TempoHardfork>,
tx_env: &'evm impl Transaction,
f: impl FnOnce() -> R,
) -> Rwhere
J: JournalTr<Database: Database> + Debug,
Generic entry point for EVM-like environments. Sets up the storage provider and executes a closure within that context.
Sourcepub fn enter_precompile<J, P, R>(
journal: &'evm mut J,
block_env: &'evm dyn Block,
cfg: &CfgEnv<TempoHardfork>,
tx_env: &'evm impl Transaction,
f: impl FnOnce(P) -> R,
) -> R
pub fn enter_precompile<J, P, R>( journal: &'evm mut J, block_env: &'evm dyn Block, cfg: &CfgEnv<TempoHardfork>, tx_env: &'evm impl Transaction, f: impl FnOnce(P) -> R, ) -> R
Entry point for a “canonical” precompile (with unique known address).
Sourceimpl StorageCtx
impl StorageCtx
Sourcefn as_hashmap(&self) -> &mut HashMapStorageProvider
Available on crate features test-utils only.
fn as_hashmap(&self) -> &mut HashMapStorageProvider
test-utils only.Returns a mutable reference to the underlying HashMapStorageProvider.
NOTE: takes a non-mutable reference because it’s internal. The mutability of the storage operation is determined by the public function.
Sourcepub fn get_account_info(&self, address: Address) -> Option<&AccountInfo>
Available on crate features test-utils only.
pub fn get_account_info(&self, address: Address) -> Option<&AccountInfo>
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn get_events(&self, address: Address) -> &Vec<LogData>
Available on crate features test-utils only.
pub fn get_events(&self, address: Address) -> &Vec<LogData>
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn set_nonce(&mut self, address: Address, nonce: u64)
Available on crate features test-utils only.
pub fn set_nonce(&mut self, address: Address, nonce: u64)
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn set_timestamp(&mut self, timestamp: U256)
Available on crate features test-utils only.
pub fn set_timestamp(&mut self, timestamp: U256)
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn set_beneficiary(&mut self, beneficiary: Address)
Available on crate features test-utils only.
pub fn set_beneficiary(&mut self, beneficiary: Address)
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn set_block_number(&mut self, block_number: u64)
Available on crate features test-utils only.
pub fn set_block_number(&mut self, block_number: u64)
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn set_spec(&mut self, spec: TempoHardfork)
Available on crate features test-utils only.
pub fn set_spec(&mut self, spec: TempoHardfork)
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn clear_transient(&mut self)
Available on crate features test-utils only.
pub fn clear_transient(&mut self)
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
Sourcepub fn clear_events(&mut self, address: Address)
Available on crate features test-utils only.
pub fn clear_events(&mut self, address: Address)
test-utils only.NOTE: assumes storage tests always use the HashMapStorageProvider
USAGE: TIP20Setup automatically clears events of the configured
contract when apply() is called, unless explicitly asked no to.
Sourcepub fn has_bytecode(&self, address: Address) -> Result<bool>
Available on crate features test-utils only.
pub fn has_bytecode(&self, address: Address) -> Result<bool>
test-utils only.Checks if a contract at the given address has bytecode deployed.